WITH Text_IO;
WITH My_Int_IO;
WITH Calendar;
PROCEDURE WorldTime IS

  TYPE Cities IS (Paris, London, Rio, Caracas, DC, 
                  Chicago, Denver, Seattle, Honolulu);

  PACKAGE City_IO IS NEW Text_IO.Enumeration_IO(Cities);

  TYPE TimeDiffs IS ARRAY (Cities) OF Integer;

  -- table of time differences from DC; modify this table if you are
  -- not located in the Eastern U.S. time zone
  Offsets : CONSTANT TimeDiffs := 
    (Paris => +6, London => +5, Rio => +2, Caracas => -1, DC => 0, 
     Chicago => -1, Denver => -2, Seattle => -3,  Honolulu => -5);

  TimeHere  : Calendar.Time;
  TimeThere : Calendar.Time;
  There     : Cities;

  FUNCTION AdjustTime(T: Calendar.Time; City: Cities;
                      OffsetTable: TimeDiffs) RETURN Calendar.Time IS

  -- given a time value, finds the corresponding time in a given time zone
  -- Pre: T, City, and OffsetTable are initialized 
  --      T represents a time in the local time zone
  --      OffsetTable represents the time zone offsets from the local zone
  -- Post: returns the adjusted time

  BEGIN -- AdjustTime

    RETURN Calendar."+"(T, Duration(Offsets(City) * 3600));

  END AdjustTime;


  PROCEDURE ReadCity(City : OUT Cities) IS

  -- reads a city name from the terminal, robustly
  -- Pre: none
  -- Post: City contains a valid city name

  BEGIN -- ReadCity

    LOOP
      BEGIN     -- exception handler block
        Text_IO.Put(Item => "Enter a city name, please > ");
        City_IO.Get(Item => City);
        EXIT;   -- good input data
      EXCEPTION -- bad input data
        WHEN Text_IO.Data_Error =>
          Text_IO.Skip_Line;
          Text_IO.Put(Item => "Invalid city name; please try again.");
          Text_IO.New_Line;
      END;      -- exception handler block
    END LOOP;

  END ReadCity;

  
  PROCEDURE DisplayTime(T: Calendar.Time) IS

  -- displays a Calendar.Time value in hh:mm:ss form
  -- Pre: T is defined
  -- Post: displays the Seconds part of T in hh:mm:ss form

    SecsPastMidnight : Calendar.Day_Duration;
    MinsPastMidnight : Natural;
    Secs             : Natural;
    Mins             : Natural;
    Hrs              : Natural;

  BEGIN -- DisplayTime

    SecsPastMidnight := Calendar.Seconds(T);
    MinsPastMidnight := Natural(Float(SecsPastMidnight)/60.0 - 0.5);
    Secs :=  
      Natural(Float(SecsPastMidnight) - 60.0 * Float(MinsPastMidnight));
    Mins :=             MinsPastMidnight REM 60;
    Hrs :=              MinsPastMidnight / 60;

    My_Int_IO.Put (Item => Hrs, Width => 1);
    Text_IO.Put (Item => ':');
    IF Mins < 10 THEN
      Text_IO.Put (Item => '0');
    END IF;
    My_Int_IO.Put (Item => Mins, Width => 1);
    Text_IO.Put (Item => ':');
    IF Secs < 10 THEN
      Text_IO.Put (Item => '0');
    END IF;
    My_Int_IO.Put (Item => Secs, Width => 1);

  END DisplayTime;

BEGIN -- WorldTime

  ReadCity(City => There);
  TimeHere := Calendar.Clock;
  TimeThere := AdjustTime(T=>TimeHere, City=>There, OffsetTable=>Offsets);

  Text_IO.Put(Item => "Current local time is ");
  DisplayTime(T => TimeHere);
  Text_IO.New_Line;
  Text_IO.Put(Item => "Current time in ");
  City_IO.Put(Item => There, Width => 1);
  Text_IO.Put(Item => " is ");
  DisplayTime(T => TimeThere);
  Text_IO.New_Line;

END WorldTime;
