WITH Text_IO;
PACKAGE BODY VStrings IS

  FUNCTION Length(S : VString) RETURN Index IS
  BEGIN
    RETURN S.CurrentLength;
  END Length;

  FUNCTION Value(S : VString) RETURN String IS
  BEGIN
    IF S.CurrentLength = 0 THEN
      RETURN "";
    ELSE
      RETURN S.StringPart(1 .. S.CurrentLength);
    END IF;
  END Value;

  FUNCTION Tail(S : VString) RETURN VString IS
  BEGIN
    IF S.CurrentLength = 0 THEN
      RAISE EmptyString;
    ELSIF S.CurrentLength = 1 THEN
      RETURN MakeVString("");
    ELSE
      RETURN MakeVString(S.StringPart(2 .. S.CurrentLength));
    END IF;
  END tail;

  FUNCTION Head(S : VString) RETURN Character IS
  BEGIN
    IF S.CurrentLength = 0 THEN
      RAISE EmptyString;
    ELSE
      RETURN S.StringPart(1);
    END IF;
  END head;

  FUNCTION IsEmpty(S : VString) RETURN Boolean IS
  BEGIN
    RETURN S.CurrentLength = 0;
  END IsEmpty;

  FUNCTION MakeVString(S : String) RETURN VString IS
    Temp : VString;
  BEGIN
    IF S'Length > MaxLength THEN
      RAISE StringOverflow;
    END IF;
    Temp.CurrentLength := S'Length;
    Temp.StringPart(1 .. S'Length) := S;
    RETURN Temp;
  END MakeVString;

  FUNCTION EmptyVString RETURN VString IS
  BEGIN
    RETURN MakeVString("");
  END EmptyVString;

  FUNCTION MakeVString(C : Character) RETURN VString IS
    Temp : VString;
  BEGIN
    Temp.CurrentLength := 1;
    Temp.StringPart(1) := C;
    RETURN Temp;
  END MakeVString;

  FUNCTION "&" (S1, S2 : VString) RETURN VString IS
    Temp : VString;
  BEGIN

    IF S1.CurrentLength + S2.CurrentLength > MaxLength THEN
      RAISE StringOverflow;
    ELSIF IsEmpty(S1) THEN
      Temp := S2;
    ELSIF IsEmpty(S2) THEN
      Temp := S1;
    ELSE
      Temp := MakeVString(Value(S1) & Value(S2));
    END IF;

    RETURN Temp;

  END "&";

  FUNCTION "&" (S1 : VString; C : Character) RETURN VString IS
  BEGIN
    RETURN S1 & MakeVString(C);
  END "&";

  FUNCTION "&" (C : Character; S1 : VString) RETURN VString IS
  BEGIN
    RETURN MakeVString(C) & S1;
  END "&";

  FUNCTION "&" (S1 : VString; S : String) RETURN VString IS
  BEGIN
    RETURN S1 & MakeVString(S);
  END "&";

  FUNCTION "&" (S : String; S1 : VString) RETURN VString IS
  BEGIN
    RETURN MakeVString(S) & S1;
  END "&";

  FUNCTION "<=" (S1, S2 : VString) RETURN Boolean IS
  BEGIN
    RETURN Value(S1) <= Value(S2);
  END "<=";

  FUNCTION "<" (S1, S2 : VString) RETURN Boolean IS
  BEGIN
    RETURN Value(S1) < Value(S2);
  END "<";

  FUNCTION ">=" (S1, S2 : VString) RETURN Boolean IS
  BEGIN
    RETURN Value(S1) >= Value(S2);
  END ">=";

  FUNCTION ">" (S1, S2 : VString) RETURN Boolean IS
  BEGIN
    RETURN Value(S1) > Value(S2);
  END ">";

  FUNCTION Locate(Sub : String; Within : VString) RETURN Index IS

    Result  : Index;
    LSub    : Index;
    LWithin : Index;

  BEGIN

    LSub := Sub'Length;
    LWithin := Within.CurrentLength;
    Result := 0;
    IF LSub > 0
      AND LWithin > 0
      AND LSub <= LWithin THEN

      FOR Start IN 1 .. (LWithin - LSub + 1) LOOP
        IF Sub = Within.StringPart(Start .. (Start + LSub - 1)) THEN
          RESULT := Start;
        END IF;
      END LOOP;
    END IF;

    RETURN Result;

  END Locate;

  FUNCTION Locate(Sub : VString; Within : VString) RETURN Index IS
  BEGIN
    RETURN Locate(Value(Sub), Within);
  END Locate;

  FUNCTION Locate(C : Character; Within : VString) RETURN Index IS

    Temp : String(1..1);

  BEGIN

    Temp(1) := C;
    RETURN Locate(Temp, Within);

  END Locate;

  FUNCTION SubString(S : VString; Start, Size : Index) RETURN VString IS
  BEGIN

    IF Start        > Length(S) OR
       Size         > Length(S) OR
       Start+Size-1 > Length(S) THEN
      RAISE InvalidArguments;
    ELSE
      RETURN MakeVString(S.StringPart(Start..Start+Size-1));
    END IF;

  END SubString;

  PROCEDURE Get_Line(File : Text_IO.File_Type; Item : OUT VString) IS

    -- reads a VString object from File, using Text_IO.Get_Line
    -- reading stops if a line terminator is encountered, or if
    -- MaxLength characters have been read.

    S     : String(1..MaxLength);
    Count : Natural;

  BEGIN

    Text_IO.Get_Line(File => File, Item => S, Last => Count);
    IF Count > 0 THEN
      Item := MakeVstring(S(1..Count));
    ELSE
      Item := EmptyVstring;
    END IF;

  END Get_Line;

  PROCEDURE Get_Line(Item : OUT VString) IS
  BEGIN
    Get_Line(File => Text_IO.Standard_Input, Item => Item);
  END Get_Line;

  PROCEDURE Put(File: Text_IO.File_Type; Item : VString) IS
  BEGIN
    Text_IO.Put(File=>File, Item=>Value(Item));
  END Put;

  PROCEDURE Put(Item : VString) IS
  BEGIN
    Put(File=>Text_IO.Standard_Output, Item=>Item);
  END Put;

END VStrings;
