WITH Text_IO;
WITH Screen;
WITH Dates;
WITH VStrings; USE Vstrings;
PROCEDURE FormLetter IS

 -- Writes a job application letter to an output file.  The data
 -- for the letter preamble is read from the keyboard; the letter
 -- body is copied from a data file to the output file.
  
   MaxLines : CONSTANT Positive := 50;
   SUBTYPE LetterRange IS Positive RANGE 1..MaxLines;
   TYPE LetterBody IS ARRAY(LetterRange) OF VStrings.VString;
   Letter       : LetterBody;
   LetterLength : LetterRange;
   LetterIn     : Text_IO.File_Type;
   LetterOut    : Text_IO.File_Type;

   OutFileName  : VStrings.VString;
   Response     : Character;
   Blanks       : CONSTANT String(1..10) := (OTHERS => ' ');

   PROCEDURE InitializeLetter IS

     InFileName : VStrings.VString;
                              
   BEGIN

     Text_IO.Put(Item => "Enter name of letter body file > ");
     VStrings.Get_Line(Item => InFileName);
     Text_IO.Open(File=>LetterIn, Mode=>Text_IO.In_File, 
                  Name=>VStrings.Value(InFileName));
     
     LetterLength := 1;
     WHILE NOT Text_IO.End_of_File (File=>LetterIn) LOOP
       VStrings.Get_Line(File=>LetterIn, Item=>Letter(LetterLength));
       LetterLength := LetterLength + 1;
     END LOOP;

     Text_IO.Close(File=>LetterIn);

   END InitializeLetter;
   

   PROCEDURE Preamble IS
   
   -- Writes a preamble for a job application letter to an output file.
   -- Pre : None
   -- Post: Writes the heading, salutation, and first sentence of a
   --       job application letter.  
     
     Title       : VStrings.VString;
     FirstName   : VStrings.VString;
     LastName    : VStrings.VString;
     Company     : VStrings.VString;
     Address     : VStrings.VString;
     CityStZip   : VStrings.VString;
     DatePadding : CONSTANT String(1..50) := (OTHERS => ' ');

   BEGIN  -- Premable   
     -- Enter all data   
     Screen.ClearScreen;
     Screen.MoveCursor(Row=>3, Column=>1);
     Text_IO.Put(Item => Blanks & "Today is             -> ");
     Dates.Put(Item => Dates.Today, Format => Dates.Full);
     Text_IO.New_Line;
     Text_IO.Put(Item => Blanks & "Employer title       -> ");
     VStrings.Get_Line(Item => Title);
     Text_IO.Put(Item => Blanks & "First Name & Initial -> ");
     VStrings.Get_Line(Item => FirstName);
     Text_IO.Put(Item => Blanks & "Employer last name   -> ");
     VStrings.Get_Line(Item => LastName);
     Text_IO.Put(Item => Blanks & "Company name         -> ");
     VStrings.Get_Line(Item => Company);
     Text_IO.Put(Item => Blanks & "Street address       -> ");
     VStrings.Get_Line(Item => Address);
     Text_IO.Put(Item => Blanks & "City, State, and ZIP -> ");
     VStrings.Get_Line(Item => CityStZip);

     -- write preamble to output file
     Text_IO.Put(File=>LetterOut,Item => DatePadding);
     Dates.Put(File=>LetterOut, Item => Dates.Today, Format=>Dates.Full);
     Text_IO.New_Line(File=>LetterOut);
     Text_IO.New_Line(File=>LetterOut);
     VStrings.Put(File=>LetterOut, 
       Item => Title & ' ' & FirstName & ' ' & LastName);
     Text_IO.New_Line(File=>LetterOut);
     VStrings.Put(File=>LetterOut, Item => Company);
     Text_IO.New_Line(File=>LetterOut);
     VStrings.Put(File=>LetterOut, Item => Address);
     Text_IO.New_Line(File=>LetterOut);
     VStrings.Put(File=>LetterOut, Item => CityStZip);
     Text_IO.New_Line(File=>LetterOut);
     Text_IO.New_Line(File=>LetterOut);
     VStrings.Put(File=>LetterOut, 
       Item => "Dear " & Title & ' ' & LastName & ':');
     Text_IO.New_Line(File=>LetterOut);
     Text_IO.New_Line(File=>LetterOut);
     VStrings.Put(File=>LetterOut, 
       Item => "     I am interested in applying for a job at " 
         & Company & '.');
     Text_IO.New_Line(File=>LetterOut);
     Text_IO.New_Line(File=>LetterOut);
     
   END Preamble;

   PROCEDURE WriteBody IS
   
   -- Copies the body of a job application letter from a data file
   -- to an output file.
   -- Pre : The output file is opened.
   -- Post: Writes the letter body to the output file.

   BEGIN -- WriteBody   
     -- Copy each line until done   
     FOR Line IN 1..LetterLength LOOP
       VStrings.Put(File=>LetterOut,Item=>Letter(Line));
       Text_IO.New_Line(File=>LetterOut);
     END LOOP;
   END WriteBody;

BEGIN -- FormLetter   

  Text_IO.Put(Item => "Enter name of letter output file > ");
  VStrings.Get_Line(Item => OutFileName);
  Text_IO.Create(File=>LetterOut, Mode=>Text_IO.Out_File, 
                Name=>VStrings.Value(OutFileName));
  InitializeLetter;
  Response := 'Y';

  LOOP
    Preamble;
    WriteBody;

    LOOP
      Screen.Beep;
      Screen.MoveCursor(Row => 20, Column => 1);
      Text_IO.Put(Blanks & "More Letters? (Y or N) ");
      Text_IO.Get(Item => Response);
      Text_IO.Skip_Line;
      EXIT WHEN (Response = 'y') OR (Response = 'Y') OR 
                (Response = 'n') OR (Response = 'N');
    END LOOP;
    -- assert: response is valid

    Text_IO.New_Page (File => LetterOut);
    EXIT WHEN (Response = 'n') OR (Response = 'N');
  END LOOP;
  -- assert: response was 'n' or 'Y'

  Text_IO.Close(File=>LetterOut);

END FormLetter;

