PACKAGE Employees IS

  -- specification for ADT package to handle Employee records

  -- constant and type definitions

  NameSize: CONSTANT Positive := 20;

  SUBTYPE IDRange IS Positive RANGE 1111..9999;
  SUBTYPE PayRange IS Float RANGE 0.0..1_000_000.0;
  SUBTYPE NameType IS String(1..NameSize);
  TYPE    GenderType IS (Female, Male);

  TYPE Employee IS PRIVATE;

  -- operations

  -- constructor

  FUNCTION MakeEmployee (ID: IDRange; 
                         Name: NameType; 
                         Gender: GenderType; 
                         NumDepend: Natural;
                         Rate: PayRange; 
                         TaxSal: PayRange)  RETURN Employee;

  -- constructs an Employee record from its field values
  -- Pre: all input parameters are defined
  -- Post: returns a value of type Employee

  -- selectors

  FUNCTION RetrieveID        (OneEmp: Employee) RETURN IDRange;
  FUNCTION RetrieveName      (OneEmp: Employee) RETURN NameType;
  FUNCTION RetrieveGender    (OneEmp: Employee) RETURN GenderType;
  FUNCTION RetrieveNumDepend (OneEmp: Employee) RETURN Natural;   
  FUNCTION RetrieveRate      (OneEmp: Employee) RETURN PayRange;
  FUNCTION RetrieveTaxSal    (OneEmp: Employee) RETURN PayRange;

  -- retrieve individual fields of an Employee record
  -- Pre: OneEmp is defined
  -- Post: each selector retrieves its desired field


  -- input/output operations

  PROCEDURE ReadEmployee (Item: OUT Employee);

  -- reads an Employee record from the terminal
  -- Pre: none
  -- Post: Item contains a record of type Employee

  PROCEDURE DisplayEmployee (Item: IN Employee);

  -- displays an Employee record on the screen
  -- Pre: Item is defined
  -- Post: displays the fields of Item on the screen

PRIVATE

  TYPE Employee IS RECORD
    ID:        IDRange := IDRange'Last;
    Name:      NameType := (OTHERS => ' ');
    Gender:    GenderType := Female;
    NumDepend: Natural := 0;
    Rate:      PayRange := 0.0;
    TaxSal:    PayRange := 0.0;
  END RECORD;

END Employees;
