WITH Text_IO;
WITH My_Flt_IO;
WITH My_Int_IO;
PACKAGE BODY Employees IS

  PACKAGE GenderType_IO IS NEW Text_IO.Enumeration_IO(Enum => GenderType);

  -- body of ADT package to handle Employee records

  -- operations

  -- constructor

  FUNCTION MakeEmployee (ID: IDRange; 
                         Name: NameType; 
                         Gender: GenderType; 
                         NumDepend: Natural;
                         Rate: PayRange; 
                         TaxSal: PayRange)  RETURN Employee IS

  -- constructs an Employee record from its field values
  -- Pre: all input parameters are defined
  -- Post: returns a value of type Employee

    TempRecord: Employee;

  BEGIN -- MakeEmployee

    TempRecord := (ID => ID, Name => Name, Gender => Gender,
                   NumDepend => NumDepend, Rate => Rate, TaxSal => TaxSal);
    RETURN TempRecord;

  END MakeEmployee;

  -- selectors
  -- retrieve individual fields of an Employee record
  -- Pre: OneEmp is defined
  -- Post: each selector retrieves its desired field

  FUNCTION RetrieveID (OneEmp: Employee) RETURN IDRange IS
  BEGIN
    RETURN OneEmp.ID;
  END RetrieveID;

  FUNCTION RetrieveName (OneEmp: Employee) RETURN NameType IS
  BEGIN
    RETURN OneEmp.Name;
  END RetrieveName;

  FUNCTION RetrieveGender (OneEmp: Employee) RETURN GenderType IS
  BEGIN
    RETURN OneEmp.Gender;
  END RetrieveGender;

  FUNCTION RetrieveNumDepend (OneEmp: Employee) RETURN Natural IS   
  BEGIN
    RETURN OneEmp.NumDepend;
  END RetrieveNumDepend;

  FUNCTION RetrieveRate (OneEmp: Employee) RETURN PayRange IS
  BEGIN
    RETURN OneEmp.Rate;
  END RetrieveRate;

  FUNCTION RetrieveTaxSal (OneEmp: Employee) RETURN PayRange IS
  BEGIN
    RETURN OneEmp.TaxSal;
  END RetrieveTaxSal;


  -- input/output operations

  PROCEDURE ReadEmployee (Item: OUT Employee) IS

  -- reads an Employee record from the terminal
  -- Pre: none
  -- Post: Item contains a record of type Employee

  BEGIN -- ReadEmployee   
  
    Text_IO.Put(Item => "ID > ");  
    My_Int_IO.Get(Item => Item.ID);  
    Text_IO.Put(Item => "Name > ");
    Text_IO.Get(Item => Item.Name);
    Text_IO.Put(Item => "Gender (Female or Male) > ");  
    GenderType_IO.Get(Item => Item.Gender);
    Text_IO.Put(Item => "Number of dependents > ");
    My_Int_IO.Get(Item => Item.NumDepend);
    Text_IO.Put(Item => "Hourly rate > ");
    My_Flt_IO.Get(Item => Item.Rate);
  
  END ReadEmployee;

  PROCEDURE DisplayEmployee (Item: IN Employee) IS

  -- displays an Employee record on the screen
  -- Pre: Item is defined
  -- Post: displays the fields of Item on the screen

  BEGIN -- stub for DisplayEmployee

    Text_IO.Put(Item => "DisplayEmployee entered.");
    Text_IO.New_Line;

  END DisplayEmployee;

END Employees;
