WITH Calendar;
WITH Text_IO;
PACKAGE Dates IS

 -- specification for package to represent calendar dates
 -- in a form convenient for reading and displaying.

  TYPE Months IS
    (Jan, Feb, Mar, Apr, May, Jun, Jul, Aug, Sep, Oct, Nov, Dec);

  TYPE Formats IS
    (Full,            -- February 7, 1991
     Short,           -- 07 FEB 91
     Numeric,         -- 2/7/91
     Julian);         -- 91038

  SUBTYPE Year_Number  IS Calendar.Year_Number;
  SUBTYPE Day_Number   IS Calendar.Day_Number;

  TYPE Date IS PRIVATE;

  -- constructors

  FUNCTION Today                       RETURN Date;
  FUNCTION Date_Of(Year  : Year_Number;
                   Month : Months;
                   Day   : Day_Number) RETURN Date;

  -- exported exception

  Date_Error : EXCEPTION;

  -- selectors

  FUNCTION Year (D: Date) RETURN Year_Number;
  FUNCTION Month(D: Date) RETURN Months;
  FUNCTION Day  (D: Date) RETURN Day_Number;


  -- input/output

  PROCEDURE Get(Item: OUT Date);
  PROCEDURE Put(Item: IN Date; Format: IN Formats);

  PROCEDURE Get(File: IN Text_IO.File_Type; Item: OUT Date);
  PROCEDURE Put(File: IN Text_IO.File_Type; 
                Item: IN Date; Format: IN Formats);

PRIVATE

  TYPE Date IS RECORD
    Month: Months;
    Day:   Calendar.Day_Number;
    Year:  Calendar.Year_Number;
  END RECORD;

END Dates;
