PACKAGE Rationals IS

-- Specification of the abstract data type for representing
-- and manipulating rational numbers.
 
  TYPE Rational IS PRIVATE;

--Operators   

  FUNCTION "/" (X : Integer; Y : Integer) RETURN Rational;
  
  -- constructor: returns a rational number in lowest terms
  -- Pre : X and Y are defined
  -- Post: returns a rational number
  --   If Y > 0, returns Reduce(X,Y)
  --   If Y < 0, returns Reduce(-X,-Y)
  --   If Y = 0, raises ZeroDenominator

  ZeroDenominator: EXCEPTION;
  
  FUNCTION Numer (R : Rational) RETURN Integer;
  FUNCTION Denom (R : Rational) RETURN Positive;

  -- selectors: return the numerator and denominator of a rational number R
  -- Pre: R is defined
  -- Post: Numer returns the numerator of R; Denom returns the denominator
    

  PROCEDURE Get (Item : OUT Rational);
  
  -- Reads a pair of integer values into rational number Item.
  -- Pre : none
  -- Post: The first integer number read is the numerator of Item;
  --       the second integer number is the denominator of Item.
  --       "/" is called to produce a rational in reduced form.


  PROCEDURE Put (Item : IN Rational);
  
  -- Displays rational number Item.
  -- Pre : Item is assigned a value.
  -- Post: displays the numerator and denominator of Item.
    
  
  FUNCTION "+"(R1 : Rational; R2 : Rational) RETURN Rational;
  FUNCTION "-"(R1 : Rational; R2 : Rational) RETURN Rational;
  FUNCTION "*"(R1 : Rational; R2 : Rational) RETURN Rational;
  FUNCTION "/"(R1 : Rational; R2 : Rational) RETURN Rational;

  -- constructors: return the rational sum, difference, product,
  --   and quotient of rational numbers R1 and B
  -- Pre : R1 and R2 are assigned values 
  -- Post: return the rational sum, difference, product, and
  --   quotient of R1 and R2.
  

  FUNCTION "<" (R1 : Rational; R2 : Rational) RETURN Boolean;
  FUNCTION ">" (R1 : Rational; R2 : Rational) RETURN Boolean;
  FUNCTION "<="(R1 : Rational; R2 : Rational) RETURN Boolean;
  FUNCTION ">="(R1 : Rational; R2 : Rational) RETURN Boolean;

  -- inquiry operators: comparison of two rational numbers
  -- Pre : R1 and R2 are assigned values 
  -- Post: return R1 < R2, R1 > R2, R1 <= R2, and R1 >= R2, respectively 
    

PRIVATE

-- R1 record of type Rational consists of a pair of integer values
-- such that the first number represents the numerator of a rational
-- number and the second number represents the denominator.
 
  TYPE Rational IS
    RECORD
      Numerator     : Integer := 0;
      Denominator   : Positive := 1;
    END RECORD; -- Rational   

END Rationals;
