WITH Text_IO;
WITH My_Int_IO;
PACKAGE BODY Rationals IS

-- Body of the abstract data type for representing
-- and manipulating rational numbers.
 
-- local function GCD 

  FUNCTION GCD(M: Positive; N: Positive) RETURN Positive IS

  -- finds the greatest common divisor of M and N
  -- Pre: M and N are defined
  -- Post: returns the GCD of M and N, by Euclid's Algorithm

    R : Natural;
    TempM: Positive;
    TempN: Positive;

  BEGIN -- GCD
    
    TempM := M;
    TempN := N;

    R := TempM REM TempN;

    WHILE R /= 0 LOOP
      TempM := TempN;
      TempN := R;
      R := TempM REM TempN;
    END LOOP;

    RETURN TempN;

  END GCD;


  -- Operators   

  FUNCTION "/" (X : Integer; Y : Integer) RETURN Rational IS
  
  -- constructor: returns a rational number in lowest terms
  -- Pre : X and Y are defined
  -- Post: returns a rational number
  --   If Y > 0, returns Reduce(X,Y)
  --   If Y < 0, returns Reduce(-X,-Y)
  --   If Y = 0, raises ZeroDenominator
    
    G: Positive;

  BEGIN

    IF Y = 0 THEN
      RAISE ZeroDenominator;
    END IF;

    IF X = 0 THEN
      RETURN (Numerator => 0, Denominator => 1);
    END IF;

    G := GCD(ABS X, ABS Y);
    IF Y > 0 THEN
      RETURN (Numerator => X/G, Denominator => Y/G);
    ELSE
      RETURN (Numerator => (-X)/G, Denominator => (-Y)/G);
    END IF;

  END "/";


  FUNCTION Numer (R : Rational) RETURN Integer IS
  BEGIN
    RETURN R.Numerator;
  END Numer;
  
  FUNCTION Denom (R : Rational) RETURN Positive IS
  BEGIN
    RETURN R.Denominator;
  END Denom;

  -- selectors: return the numerator and denominator of a rational number R
  -- Pre: R is defined
  -- Post: Numer returns the numerator of R; Denom returns the denominator
    

  PROCEDURE Get (Item : OUT Rational) IS
  
  -- Reads a pair of integer values into rational number Item.
  -- Pre : none
  -- Post: The first integer number read is the numerator of Item;
  --       the second integer number is the denominator of Item.
  --       "/" is called to produce a rational in reduced form.

    N: Integer;
    D: Integer;

  BEGIN -- Get

    My_Int_IO.Get(Item => N);
    My_Int_IO.Get(Item => D);
    Item := N/D;

  END Get;
    

  PROCEDURE Put (Item : IN Rational) IS
  
  -- Displays rational number Item.
  -- Pre : Item is assigned a value.
  -- Post: displays the numerator and denominator of Item.

  BEGIN -- Put

    My_Int_IO.Put(Item => Numer(Item), Width => 1);
    Text_IO.Put(Item => '/');
    My_Int_IO.Put(Item => Denom(Item), Width => 1);

  END Put;
    
  FUNCTION "+"(R1 : Rational; R2 : Rational) RETURN Rational IS
    N: Integer;
    D: Positive;
  BEGIN
    N := Numer(R1) * Denom(R2) + Numer(R2) * Denom(R1);
    D := Denom(R1) * Denom(R2);
    RETURN N/D;  -- compiler will use fraction constructor here!
  END "+";
  
  FUNCTION "*"(R1 : Rational; R2 : Rational) RETURN Rational IS
    N: Integer;
    D: Positive;
  BEGIN
    N := Numer(R1) * Numer(R2);
    D := Denom(R1) * Denom(R2);
    RETURN N/D;  -- compiler will use fraction constructor here!
  END "*";

  FUNCTION "-"(R1 : Rational; R2 : Rational) RETURN Rational IS
  BEGIN -- stub
    RETURN 1/1;
  END "-";

  FUNCTION "/"(R1 : Rational; R2 : Rational) RETURN Rational IS
  BEGIN -- stub
    RETURN 1/1;
  END "/";

  -- constructors: return the rational sum, difference, product,
  --   and quotient of rational numbers R1 and B
  -- Pre : R1 and R2 are assigned values 
  -- Post: return the rational sum, difference, product, and
  --   quotient of R1 and R2.
  

  FUNCTION "<" (R1 : Rational; R2 : Rational) RETURN Boolean IS
  BEGIN
    RETURN Numer(R1) * Denom(R2) < Numer(R2) * Denom(R1);
  END "<";

  FUNCTION ">" (R1 : Rational; R2 : Rational) RETURN Boolean IS
  BEGIN -- stub
    RETURN True;
  END ">";

  FUNCTION "<=" (R1 : Rational; R2 : Rational) RETURN Boolean IS
  BEGIN -- stub
    RETURN True;
  END "<=";

  FUNCTION ">=" (R1 : Rational; R2 : Rational) RETURN Boolean IS
  BEGIN -- stub
    RETURN True;
  END ">=";

  -- inquiry operators: comparison of two rational numbers
  -- Pre : R1 and R2 are assigned values 
  -- Post: return R1 < R2, R1 > R2, R1 <= R2, and R1 >= R2, respectively 
    

END Rationals;
