PACKAGE Geometry IS
  
-- Defines an abstract data type for a geometric figure.  
-- Operators include constructors for rectangles, circles, and squares,
-- selectors for area and perimeter, and input/output operations
  
-- Data Types

  SUBTYPE NonNegFloat IS Float RANGE 0.0 .. Float'Last;
  TYPE FigKind IS (Rect, Square, Circle);

  TYPE Figure (FigShape : FigKind := Rect) IS PRIVATE;

-- Operators  

  FUNCTION MakeRect (Width, Height : NonNegFloat) RETURN Figure;

  -- constructor: creates a rectangular figure from its height and width
  -- Pre : Width and Height are defined
  -- Post: returns a figure of type Rect


  FUNCTION MakeCircle (Radius : NonNegFloat) RETURN Figure;

  -- constructor: creates a circular figure from its radius
  -- Pre : Radius is defined
  -- Post: returns a figure of type Circle


  FUNCTION MakeSquare (Side : NonNegFloat) RETURN Figure;

  -- constructor: creates a square figure from its side
  -- Pre : Side is defined
  -- Post: returns a figure of type Side


  PROCEDURE ReadFigure (OneFig : OUT Figure);
  
  -- Enters data into OneFig.
  -- Pre : None
  -- Post: The discriminant and characteristics of OneFig are defined.
    

  FUNCTION Perimeter (OneFig : Figure) RETURN NonNegFloat;
  
  -- selector: returns perimeter of OneFig.
  -- Pre : The discriminant and characteristics of OneFig are defined.
  -- Post: Returns perimeter
    

  FUNCTION Area (OneFig : Figure) RETURN NonNegFloat;
  
  -- selector: returns Area of OneFig.
  -- Pre : The discriminant and characteristics of OneFig are defined.
  -- Post: Returns area of OneFig.
    

  PROCEDURE DisplayFigure (OneFig : IN Figure);
  --
  -- Displays the characteristics of OneFig.
  -- Pre : All fields of OneFig are defined.
  -- Post: Displays each field of OneFig.

PRIVATE

  TYPE Figure (FigShape : FigKind := Rect) IS RECORD
    Area : NonNegFloat := 0.0;
    Perimeter : NonNegFloat := 0.0;
    CASE FigShape IS
      WHEN Rect =>
        Width : NonNegFloat := 0.0;
        Height : NonNegFloat := 0.0;
      WHEN Square =>
        Side : NonNegFloat := 0.0;
      WHEN Circle =>
        Radius : NonNegFloat := 0.0;
    END CASE;  
  END RECORD; -- Figure  

END Geometry;
