WITH Dates;
PACKAGE AnnualTotals IS

-- Package Specification for abstract data type for a table of sales
-- data summarized by year.  The operations performed
-- include: displaying the sales data, finding the maximum yearly
-- sales, and plotting a bar graph by year.
 
  -- Data Types

  MinYear : CONSTANT Dates.Year_Number := 1985;
  MaxYear : CONSTANT Dates.Year_Number := 1990;
  SUBTYPE YearRange IS Dates.Year_Number RANGE MinYear..MaxYear;

  TYPE YearArray IS ARRAY (YearRange) OF Float;
  TYPE YearSums  IS RECORD -- all fields are initialized by default
    FirstYear : YearRange := MinYear;
    LastYear  : YearRange := MaxYear;
    Sums      : YearArray := (OTHERS => 0.0);
  END RECORD; -- YearSums 

  -- operators  

  PROCEDURE DisplayYear (SumByYear : IN YearSums);
  
  -- Displays sales totals by year. Sums are retrieved from array SumByYear.
  -- Pre : SumByYear.Sums(i) contains sales totals for year i.
  -- Post: SumByYear.First and SumByYear.Last are defined and elements
  --      SumByYear.Sums(First) through SumByYear.Sums(Last) are displayed.
   
  PROCEDURE MaxByYear (SumByYear   : IN YearSums;
                       LargeAmount : OUT Float;
                       LargeYear   : OUT YearRange);
  
  -- Finds the largest value in array SumByYear.Sums
  -- Pre : Records SumByYear is defined
  -- Post: Returns the largest value in array SumByYear.Sums through
  --       LargeAmount, and its corresponding year through LargeYear.
   
  PROCEDURE GraphYear (SumByYear: IN YearSums);
  
  -- Displays a graph of array SumByYear.Sums.
  -- Pre : Record SumByYear is defined.
  -- Post: Displays a bar graph such that the size of the bar drawn for
  --      each element of array SumByYear.Sums is proprtional to its value.
  -- calls: MaxByYear
   
END AnnualTotals;
