WITH Dates;
PACKAGE MonthlyTotals IS

-- Package Specification for abstract data type for a table of sales
-- data summarized by month.  The operations performed
-- include: displaying the sales data, finding the maximum monthly
-- sales, and plotting a bar graph by month.
 
  -- Data Types

  TYPE MonthArray IS ARRAY (Dates.Months) OF Float;
  TYPE MonthSums  IS RECORD -- all fields are initialized by default
    FirstMonth: Dates.Months := Dates.Months'First;
    LastMonth : Dates.Months := Dates.Months'Last;
    Sums      : MonthArray := (OTHERS => 0.0);
  END RECORD; -- MonthSums 

  -- operators  

  PROCEDURE DisplayMonth (SumByMonth : IN MonthSums);
  
  -- Displays sales totals by month. Sums are retrieved from array SumByMonth.
  -- Pre : SumByMonth.Sums(i) contains sales totals for month i.
  -- Post: SumByMonth.First and SumByMonth.Last are defined and elements
  --      SumByMonth.Sums(First) through SumByMonth.Sums(Last) are displayed.
   
  PROCEDURE MaxByMonth (SumByMonth : IN MonthSums;
                       LargeAmount : OUT Float;
                       LargeMonth  : OUT Dates.Months);
  
  -- Finds the largest value in array SumByMonth.Sums
  -- Pre : Records SumByMonth is defined
  -- Post: Returns the largest value in array SumByMonth.Sums through
  --       LargeAmount, and its corresponding month through LargeMonth.
   
  PROCEDURE GraphMonth (SumByMonth: IN MonthSums);
  
  -- Displays a graph of array SumByMonth.Sums.
  -- Pre : Record SumByMonth is defined.
  -- Post: Displays a bar graph such that the size of the bar drawn for
  --      each element of array SumByMonth.Sums is proprtional to its value.
  -- calls: MaxByMonth
   
END MonthlyTotals;
