WITH Text_IO;
WITH My_Int_IO;
WITH My_Flt_IO;
WITH Dates;
PACKAGE BODY AnnualTotals IS

-- Package Specification for abstract data type for a table of sales
-- data organized by year and month.  The operations performed
-- include: reading the sales data, displaying the data,
-- tabulating sums by year or month.
 
  -- operators  

  PROCEDURE DisplayYear (SumByYear : IN YearSums) IS
  
  -- Displays sales totals by year. Sums are retrieved from array SumByYear.
  -- Pre : SumByYear.Sums(i) contains sales totals for year i.
  -- Post: SumByYear.FirstYear and SumByYear.LastYear are defined and elements
  --       SumByYear.Sums(FirstYear) through SumByYear.Sums(LastYear) 
  --       are displayed.

  BEGIN -- stub for DisplayYear

    Text_IO.Put(Item => "Procedure DisplayByYear entered.");
    Text_IO.New_Line;

  END DisplayYear;
   
  PROCEDURE MaxByYear (SumByYear   : IN YearSums;
                       LargeAmount : OUT Float;
                       LargeYear   : OUT YearRange) IS
  
  -- Finds the largest value in array SumByYear.Sums
  -- Pre : Records SumByYear is defined
  -- Post: Returns the largest value in array SumByYear.Sums through
  --       LargeAmount, and its corresponding year through LargeYear.

    TempMax  : Float;
    TempYear : YearRange;

  BEGIN
   
    TempYear := SumByYear.FirstYear;
    TempMax  := SumByYear.Sums(SumByYear.FirstYear);
    
    -- find largest value in array field Sums
    FOR CurYear IN SumByYear.FirstYear+1 .. SumByYear.LastYear LOOP
      -- invariant:
      --   TempMax contains largest amount so far and 
      --   SumByYear.Sums(TempYear) is equal to TempMax
      
      IF SumByYear.Sums(CurYear) > TempMax THEN
        -- save largest amount so far.
        TempMax := SumByYear.Sums(CurYear);
        TempYear := CurYear;
      END IF;

    END LOOP;

    LargeAmount := TempMax;
    LargeYear   := TempYear;

  END MaxByYear;

  PROCEDURE GraphYear (SumByYear: IN YearSums) IS
  
  -- Displays a graph of array SumByYear.Sums.
  -- Pre : Record SumByYear is defined.
  -- Post: Displays a bar graph such that the size of the bar drawn for
  --      each element of array SumByYear.Sums is proprtional to its value.
  -- calls: MaxByYear
   
    Star        : CONSTANT Character := '*';
    ScreenWidth : CONSTANT Positive := 50;

    LargeAmount : Float;      -- largest value plotted
    Increment   : Float;      -- amount represented by each point
    PlotVal     : Float;
    LargeYear   : YearRange;

  BEGIN -- GraphYear
    -- define scale for horizontal axis
    MaxByYear (SumByYear, LargeAmount, LargeYear);
    Increment := LargeAmount / Float(ScreenWidth);

    -- plot bar graph
    Text_IO.New_Line;
    Text_IO.Put (Item => "Year|     Sales in Dollars"); -- heading
    Text_IO.New_Line;
    
    FOR CurYear IN SumByYear.FirstYear .. SumByYear.LastYear LOOP
      My_Int_IO.Put(Item => CurYear, Width => 4);
      Text_IO.Put(Item => '|');
      -- plot points until value plotted exceeds element value
      IF SumByYear.Sums(CurYear) /= 0.0 THEN
        PlotVal := Increment;
        WHILE PlotVal <= SumByYear.Sums(CurYear) LOOP
          Text_IO.Put(Item => Star);
          PlotVal := PlotVal + Increment;
        END LOOP;
      END IF;
      Text_IO.New_Line;
    END LOOP;

    -- draw horizontal scale
    Text_IO.Put(Item => "0.00^         ^         ^         ^         ^");
    Text_IO.Put(Item =>      "         ^");
    My_Flt_IO.Put(Item => LargeAmount, Fore=>5, Aft=>2, Exp=>0);
    Text_IO.New_Line;
    Text_IO.Put(Item => "     Each point represents ");
    My_Flt_IO.Put(Item => Increment, Fore=>5, Aft=>2, Exp=>0);
    Text_IO.New_Line;

  END GraphYear;

END AnnualTotals;
