WITH Text_IO;
WITH My_Int_IO;
WITH My_Flt_IO;
WITH GenericSwapSort;
PROCEDURE TestGenericSort IS

  TYPE    Color IS (Red, Orange, Yellow, Green, Blue, Violet);
  TYPE    CrayonVector IS ARRAY(Color RANGE <>) of Natural;

  PACKAGE Color_IO IS NEW Text_IO.Enumeration_IO(Enum => Color);

  SUBTYPE Index IS Integer RANGE 1..10;
  TYPE    FloatVector IS ARRAY(Index RANGE <>) OF Float;

  V1 : FloatVector(1..10);
  V2 : CrayonVector(Red..Violet);

  -- local procedures to display the contents of a vector

  PROCEDURE DisplayCrayonVector (V: CrayonVector) IS
  BEGIN
    FOR Count IN V'First..V'Last LOOP
      Color_IO.Put (Item=>Count,    Width=>6);
      My_Int_IO.Put(Item=>V(Count), Width=>4);
      Text_IO.New_Line;
    END LOOP;
    Text_IO.New_Line;
  END DisplayCrayonVector;

  PROCEDURE DisplayFloatVector (V: FloatVector) IS
  BEGIN
    FOR Count IN V'First..V'Last LOOP
      My_Flt_IO.Put(Item=>V(Count), Fore=>4, Aft=>2, Exp=>0);
    END LOOP;
    Text_IO.New_Line;
  END DisplayFloatVector;

  -- two instances of GenericSwapSort for Float vectors;
  -- the first sorts in increasing order, the second in decreasing order

  PROCEDURE SortUpFloat IS NEW GenericSwapSort
    (ElementType => Float,
     IndexType   => Index,
     ListType    => FloatVector,
     Compare     => "<");

  PROCEDURE SortDownFloat IS NEW GenericSwapSort
    (ElementType => Float,
     IndexType   => Index,
     ListType    => FloatVector,
     Compare     => ">");

  -- two instances of GenericSwapSort for Float vectors;
  -- the first sorts in increasing order, the second in decreasing order

  PROCEDURE SortUpCrayon IS NEW GenericSwapSort
    (ElementType => Natural,
     IndexType   => Color,
     ListType    => CrayonVector,
     Compare     => "<");

  PROCEDURE SortDownCrayon IS NEW GenericSwapSort
    (ElementType => Natural,
     IndexType   => Color,
     ListType    => CrayonVector,
     Compare     => ">");

BEGIN -- TestGenericSort

  V1 := (0.7, 1.5, 6.9, -3.2, 0.0, 5.1, 2.0, 7.3, 2.2, -5.9);
  Text_IO.New_Line;
  Text_IO.Put(Item=> "Testing GenericSwapSort for float vectors");
  Text_IO.New_Line;
  Text_IO.Put(Item=> "Here is the vector before sorting.");
  Text_IO.New_Line;
  DisplayFloatVector(V => V1);
  Text_IO.New_Line;
  
  SortUpFloat(List => V1);
  Text_IO.Put(Item=> "Here is the vector after upward sorting.");
  Text_IO.New_Line;
  DisplayFloatVector(V => V1);
  Text_IO.New_Line;

  SortDownFloat(List => V1);
  Text_IO.Put(Item=> "Here is the vector after downward sorting.");
  Text_IO.New_Line;
  DisplayFloatVector(V => V1);
  Text_IO.New_Line;

  V2 := (0, 100, 23, 27, 15, 94);
  Text_IO.New_Line;
  Text_IO.Put(Item=> "Testing GenericSwapSort for crayon vectors");
  Text_IO.New_Line;
  Text_IO.Put(Item=> "Here is the vector before sorting.");
  Text_IO.New_Line;
  DisplayCrayonVector(V => V2);
  Text_IO.New_Line;
  
  SortUpCrayon(List => V2);
  Text_IO.Put(Item=> "Here is the vector after upward sorting.");
  Text_IO.New_Line;
  DisplayCrayonVector(V => V2);
  Text_IO.New_Line;

  SortDownCrayon(List => V2);
  Text_IO.Put(Item=> "Here is the vector after downward sorting.");
  Text_IO.New_Line;
  DisplayCrayonVector(V => V2);

END TestGenericSort;
