GENERIC

  TYPE ValueType IS PRIVATE;
  TYPE IndexType IS (<>);

  WITH FUNCTION "+"(L,R: ValueType) RETURN ValueType;
  WITH FUNCTION "*"(L,R: ValueType) RETURN ValueType;

  Zero: ValueType;

PACKAGE GenericVectors IS

  -- generic specification for vector arithmetic package
 
  TYPE Vector IS ARRAY(IndexType RANGE <>) OF ValueType;
 
  -- exported exception, raised if two vectors are not conformable
  -- (i.e., have different bounds)

  Bounds_Error : EXCEPTION;
 
  FUNCTION "+" (K : ValueType; Right : Vector) RETURN Vector;

  -- adding a scalar to a vector
  -- Pre: K and Right are defined
  -- Post: returns the sum of the vector and the scalar
  --   Result(i) := K + Right(i)
 
 
  FUNCTION "*" (K : ValueType; Right : Vector) RETURN Vector;
 
  -- multiplying a vector by a scalar
  -- Pre: K and Right are defined
  -- Post: returns the product of the vector and the scalar
  --   Result(i) := K * Right(i)

 
  FUNCTION "*" (Left, Right : Vector) RETURN ValueType;
 
  -- finds the "inner" or "dot" product of two vectors
  -- Pre: Left and Right are defined and have the same bounds
  -- Post: returns the inner product of Left and Right

 
  FUNCTION "+" (Left, Right : Vector) RETURN Vector;
 
  -- finds the sum of two vectors
  -- Pre: Left and Right are defined and have the same bounds
  -- Post: returns the sum of Left and Right
  --    Result(i) := Left(i) + Right(i)
 

END GenericVectors;
