WITH Text_IO;
WITH My_Int_IO;
PROCEDURE TestSwapSort IS

  TYPE    Color IS (Red, Orange, Yellow, Green, Blue, Violet);
  TYPE    CrayonVector IS ARRAY(Color RANGE <>) of Natural;

  PACKAGE Color_IO IS NEW Text_IO.Enumeration_IO(Enum => Color);

  V1 : CrayonVector(Red..Green);
  V2 : CrayonVector(Yellow..Violet);
  V3 : CrayonVector(Red..Violet);

  -- local procedure to display the contents of a vector

  PROCEDURE DisplayCrayonVector (V: CrayonVector) IS

  -- display contents of a list represented as an unconstrained array
  -- Pre: V is defined
  -- Post: display all elements in the vector

  BEGIN -- DisplayCrayonVector

    FOR Count IN V'Range LOOP
      Color_IO.Put (Item=>Count,    Width=>6);
      My_Int_IO.Put(Item=>V(Count), Width=>4);
      Text_IO.New_Line;
    END LOOP;
    Text_IO.New_Line;

  END DisplayCrayonVector;

  PROCEDURE Exchange(Value1, Value2: IN OUT Natural) IS

  -- exchange two values
  -- Pre: Value1 and Value2 are defined
  -- Post: Value1 and Value2 are interchanged

    TempValue: Natural;

  BEGIN -- Exchange

    TempValue := Value1;
    Value1    := Value2;
    Value2    := TempValue;

  END Exchange;


  PROCEDURE SwapSort(List: IN OUT CrayonVector) IS

  -- sort elements of a list in ascending order
  -- Pre: List is defined
  -- Post: elements of List are arranged in ascending order

  BEGIN -- SwapSort

    FOR PositionToFill IN List'First..List'Last LOOP

      -- Store in List(PositionToFill) the smallest element remaining
      -- in the subarray List(PositionToFill..List'Last)

      FOR ItemToCompare IN PositionToFill..List'Last LOOP
        IF List(ItemToCompare) < List(PositionToFill) THEN
          Exchange(List(PositionToFill),List(ItemToCompare));
        END IF;
      END LOOP;
      -- assert: element at List(PositionToFill) is smallest in subarray

    END LOOP;

  END SwapSort;


BEGIN -- TestSwapSort

  V1 := (0, 100, 23, 27);
  V2 := (100, 23, 27, 15);
  V3 := (0, 100, 23, 27, 15, 94);
  Text_IO.New_Line;
  Text_IO.Put(Item=> "Testing SwapSort for crayon vectors");
  Text_IO.New_Line;
  Text_IO.Put(Item=> "Here is the vector V1 before sorting.");
  Text_IO.New_Line;
  DisplayCrayonVector(V => V1);
  Text_IO.New_Line;
  
  SwapSort(List => V1);
  Text_IO.Put(Item=> "Here is the vector after upward sorting.");
  Text_IO.New_Line;
  DisplayCrayonVector(V => V1);
  Text_IO.New_Line;

  Text_IO.Put(Item=> "Here is the vector V2 before sorting.");
  Text_IO.New_Line;
  DisplayCrayonVector(V => V2);
  Text_IO.New_Line;
  
  SwapSort(List => V2);
  Text_IO.Put(Item=> "Here is the vector after upward sorting.");
  Text_IO.New_Line;
  DisplayCrayonVector(V => V2);
  Text_IO.New_Line;

  Text_IO.Put(Item=> "Here is the vector V3 before sorting.");
  Text_IO.New_Line;
  DisplayCrayonVector(V => V3);
  Text_IO.New_Line;
  
  SwapSort(List => V3);
  Text_IO.Put(Item=> "Here is the vector after upward sorting.");
  Text_IO.New_Line;
  DisplayCrayonVector(V => V3);
  Text_IO.New_Line;

END TestSwapSort;
