PACKAGE Vectors IS

  -- specification for vector arithmetic package
 
  TYPE Vector IS ARRAY(Integer RANGE <>) OF Integer;
 
  -- exported exception, raised if two vectors are not conformable
  -- (i.e., have different bounds)

  Bounds_Error : EXCEPTION;
 
  FUNCTION "+" (K : Integer; Right : Vector) RETURN Vector;

  -- adding a scalar to a vector
  -- Pre: K and Right are defined
  -- Post: returns the sum of the vector and the scalar
  --   Result(i) := K + Right(i)
 
 
  FUNCTION "*" (K : Integer; Right : Vector) RETURN Vector;
 
  -- multiplying a vector by a scalar
  -- Pre: K and Right are defined
  -- Post: returns the product of the vector and the scalar
  --   Result(i) := K * Right(i)

 
  FUNCTION "*" (Left, Right : Vector) RETURN Integer;
 
  -- finds the "inner" or "dot" product of two vectors
  -- Pre: Left and Right are defined and have the same bounds
  -- Post: returns the inner product of Left and Right

 
  FUNCTION "+" (Left, Right : Vector) RETURN Vector;
 
  -- finds the sum of two vectors
  -- Pre: Left and Right are defined and have the same bounds
  -- Post: returns the sum of Left and Right
  --    result(i) := Left(i) + Right(i)
 

END Vectors;
