PACKAGE BODY Vectors IS

  -- body of Vectors package
 
  FUNCTION "+" (K : Integer; Right : Vector) RETURN Vector IS

  -- adding a scalar to a vector
  -- Pre: K and Right are defined
  -- Post: returns the sum of the vector and the scalar
  --   Result(i) := K + Right(i)
 
    Result : Vector(Right'Range);
 
  BEGIN -- "+"
 
    FOR R IN Right'Range LOOP
      Result(R) := K + Right(R);
    END LOOP;
 
    RETURN Result;
 
  END "+";
 
  FUNCTION "*" (K : Integer; Right : Vector) RETURN Vector IS
 
  -- multiplying a vector by a scalar
  -- Pre: K and Right are defined
  -- Post: returns the product of the vector and the scalar
  --   Result(i) := K * Right(i)

    Result : Vector(Right'Range);
 
  BEGIN -- "*"
 
    FOR R IN Right'Range LOOP
      Result(R) := K * Right(R);
    END LOOP;
 
    RETURN Result;
 
  END "*";
 
 
  FUNCTION "*" (Left, Right : Vector) RETURN Integer IS
 
  -- finds the "inner" or "dot" product of two vectors
  -- Pre: Left and Right are defined and have the same bounds
  -- Post: returns the inner product of Left and Right

    Sum : Integer;
 
  BEGIN -- "*"
 
  -- First check for conformability
    IF Left'First = Right'First AND 
       Left'Last  = Right'Last  THEN
  
    -- if conformable, go on to compute
      Sum := 0;
      FOR R IN Left'Range LOOP
	Sum := Sum + Left(R) * Right(R);
      END LOOP;
 
      RETURN Sum;
    ELSE
      RAISE Bounds_Error;
    END IF;

  END "*";
 
  FUNCTION "+" (Left, Right : Vector) RETURN Vector IS
 
  -- finds the sum of two vectors
  -- Pre: Left and Right are defined and have the same bounds
  -- Post: returns the sum of Left and Right
  --    result(i) := Left(i) + Right(i)
 
    Result : Vector(Left'Range);
 
  BEGIN -- "+"
 
  -- First check for conformability
    IF Left'First = Right'First AND 
       Left'Last  = Right'Last  THEN
  
    -- if conformable, go on to compute
      FOR R IN Left'Range LOOP
	Result(R) := Left(R) + Right(R);
      END LOOP;
 
      RETURN Result;

    ELSE
      RAISE Bounds_Error;
    END IF;
 
  END "+";
 

END Vectors;
