WITH Text_IO;
WITH My_Int_IO;
PROCEDURE TestBlobCount IS
  
  TYPE Fill IS (Empty, Filled);
  TYPE BlobArray IS ARRAY (Integer RANGE <>,Integer RANGE <>) OF Fill;

  Test: BlobArray(1..5,1..5);

  FUNCTION BlobCount(Grid : BlobArray; X, Y: Integer) RETURN Natural IS
  -- Counts the number of filled cells in the blob containing
  -- point (X, Y).
  -- Pre : Blob array Grid and point (X,Y) are defined.
  -- Post: Returns the size of the blob containing point (X, Y).
  --       Resets the status of each cell in the blob to Empty.

    CopyOfGrid : BlobArray(Grid'Range(1),Grid'Range(2)); 
                             -- because functions can't modify
                             -- their parameters, in Ada

    FUNCTION Blob (X, Y : Integer) RETURN Natural IS
      
    -- Inner function that performs the counting operation for BlobCount
    -- Pre : Global array CopyOfGrid and point (X,Y) are defined.
    -- Post: Returns the size of the blob containing point (X, Y).
    --       Resets the status of each cell in the blob to Empty.
      
    Result: Natural;

    BEGIN -- Blob   

      IF (X NOT IN CopyOfGrid'Range(1)) OR
         (Y NOT IN CopyOfGrid'Range(2)) THEN
        Result := 0;                        -- cell not in grid   
      ELSIF CopyOfGrid(X, Y) = Empty THEN
        Result := 0;                        -- cell is empty   
      ELSE -- cell is filled   
        -- recursive step   
        CopyOfGrid(X, Y) := Empty;
        Result := 1 + Blob(X-1, Y+1) + Blob(X, Y+1) +
                      Blob(X+1, Y+1) + Blob(X+1, Y) +
                      Blob(X+1, Y-1) + Blob(X, Y-1) +
                      Blob(X-1, Y-1) + Blob(X-1, Y);
      END IF;

      RETURN Result;

    END Blob;

  BEGIN

    CopyOfGrid := Grid;
    RETURN Blob(X,Y);

  END BlobCount;

BEGIN -- TestBlobCount

  Test := ((Empty,  Filled, Empty,  Empty,  Filled),
           (Filled, Empty,  Empty,  Filled, Filled),
           (Empty,  Empty,  Filled, Filled, Empty ),
           (Filled, Empty,  Empty,  Empty,  Empty ),
           (Filled, Filled, Filled, Empty,  Empty ));

  Text_IO.Put(Item => "BlobCount(3,4) is ");
  My_Int_IO.Put(Item => BlobCount(Test,3,4), Width => 1);
  Text_IO.New_Line;

  Text_IO.Put(Item => "BlobCount(1,2) is ");
  My_Int_IO.Put(Item => BlobCount(Test,1,2), Width => 1);
  Text_IO.New_Line;

  Text_IO.Put(Item => "BlobCount(5,5) is ");
  My_Int_IO.Put(Item => BlobCount(Test,5,5), Width => 1);
  Text_IO.New_Line;

  Text_IO.Put(Item => "BlobCount(5,1) is ");
  My_Int_IO.Put(Item => BlobCount(Test,5,1), Width => 1);
  Text_IO.New_Line;

END TestBlobCount;
