WITH Text_IO;
WITH My_Int_IO;
PROCEDURE TestBinarySearch IS

  SUBTYPE KeyType IS Integer;
  TYPE SearchArray IS ARRAY(Positive RANGE <>) OF KeyType;
  Test: SearchArray(1..9);

  FUNCTION BinarySearch (Slice: SearchArray; Target: KeyType) RETURN Natural IS
  
  -- Performs a recursive binary search of an ordered array of 
  -- keys with bounds Slice'First..Slice'Last.
  -- Pre : Target and Slice are defined.
  --     0 < Slice'First <= Slice'Last
  -- Post: Returns the subscript of Target if found in array Slice;
  --     otherwise, returns 0
  
    Middle : Integer;      -- the subscript of the middle element   

  BEGIN -- BinarySearch   

    Middle := (Slice'First + Slice'Last) / 2; -- define Middle   

    -- Determine if Target is found or missing or redefine subarray.   

    IF Slice'Length = 0 THEN
      RETURN 0;                           -- stopping case: Target missing   
    ELSIF Slice(Middle) = Target THEN
      RETURN Middle;                      -- stopping case: Target found   
    ELSIF Slice(Middle) > Target THEN     -- search lower subarray   
      RETURN BinarySearch (Slice(Slice'First..Middle-1),Target);   
    ELSE                                  -- search upper subarray   
      RETURN BinarySearch (Slice(Middle+1..Slice'Last),Target);  
    END IF;

  END BinarySearch;

BEGIN -- TestBinarySearch

  Test := (20,35,37,40,45,50,51,55,67);

  Text_IO.Put(Item => "BinarySearch(Test,35) is");
  My_Int_IO.Put(Item => BinarySearch(Test,35));
  Text_IO.New_Line;
  Text_IO.Put(Item => "BinarySearch(Test,19) is");
  My_Int_IO.Put(Item => BinarySearch(Test,19));
  Text_IO.New_Line;
  Text_IO.Put(Item => "BinarySearch(Test,75) is");
  My_Int_IO.Put(Item => BinarySearch(Test,75));
  Text_IO.New_Line;
  Text_IO.Put(Item => "BinarySearch(Test,20) is");
  My_Int_IO.Put(Item => BinarySearch(Test,20));
  Text_IO.New_Line;
  Text_IO.Put(Item => "BinarySearch(Test,67) is");
  My_Int_IO.Put(Item => BinarySearch(Test,67));
  Text_IO.New_Line;
  Text_IO.Put(Item => "BinarySearch(Test,54) is");
  My_Int_IO.Put(Item => BinarySearch(Test,54));
  Text_IO.New_Line;

END TestBinarySearch;
