WITH Text_IO;
WITH My_Int_IO;
PROCEDURE TestSelect IS

  TYPE IntArray IS ARRAY(Integer RANGE <>) OF Integer;
  Test: IntArray(-4..5);


  PROCEDURE PlaceLargest (X : IN OUT IntArray) IS
    
    -- Finds the largest element in array slice X(X'First..X'Last) 
    -- and exchanges it with the element at X(X'Last).
    -- Pre : Array X is defined and X'First <= X'Last.
    -- Post: X'Last contains the largest value.
    
    Temp :     Integer;  -- temporary copy for exchange   
    MaxIndex : Integer;  -- index of largest so far   

  BEGIN -- PlaceLargest   

    MaxIndex := X'Last;  -- assume X(X'Last) is largest   
    FOR j IN REVERSE X'First..X'Last-1 LOOP
      IF X(j) > X(MaxIndex) THEN
        MaxIndex := j;   -- X(j) is largest so far   
      END  IF;
    END LOOP;

    -- assertion: MaxIndex is subscript of largest element   
    IF MaxIndex /= X'Last THEN -- exchange X(X'Last) and X(MaxIndex)   
      Temp := X(X'Last);  
      X(X'Last) := X(MaxIndex);  
      X(MaxIndex) := Temp;
    END IF;

  END PlaceLargest;

  PROCEDURE SelectSort (X : IN OUT IntArray) IS
    
    -- Sorts an integer array slice X with bounds X'First and X'Last.
    -- Pre : Array X is defined and X'First <= X'Last.
    -- Post: The array elements are in numerical order.
    
  BEGIN -- SelectSort   

    IF X'First = X'Last THEN
      RETURN;                  -- stopping case   
    ELSIF X'First > X'Last THEN
      Text_IO.Put(Item => "Invalid slice bounds for SelectSort");
      Text_IO.New_Line;
    ELSE
      -- recursive step: place largest value in X(X'Last), 
      -- then sort the slice X(X'First..X'Last-1)
      PlaceLargest (X => X);
      SelectSort (X => X(X'First..X'Last-1));
    END IF;

  END SelectSort;

BEGIN

  Test := (1,19,3,17,5,15,7,13,9,11);
  Text_IO.Put(Item => "Before sorting: ");
  FOR Count IN Test'First .. Test'Last LOOP
    My_Int_IO.Put(Item => Test(Count), Width => 3);
  END LOOP;
  Text_IO.New_Line;

  SelectSort(X => Test);
  Text_IO.Put(Item => "After  sorting: ");
  FOR Count IN Test'First .. Test'Last LOOP
    My_Int_IO.Put(Item => Test(Count), Width => 3);
  END LOOP;
  Text_IO.New_Line;

END TestSelect;
