PROCEDURE Partition (Table : IN OUT IntArray;
                     PivIndex : OUT Integer) IS

 -- Partitions the array slice Table with bounds Table'First and
 -- Table'Last into two subarrays.
 -- Pre : Table is defined and T'First <= T'Last.
 -- Post: PivIndex is defined such that all values less than or equal
 --    to Table(PivIndex) have subscripts < PivIndex; all values
 --    greater than Table(PivIndex) have subscripts > PivIndex.

  Pivot : Integer;            -- the pivot value   
  Up    : Integer;            -- pointer to values > Pivot   
  Down  : Integer;            -- pointer to values <= Pivot   

BEGIN  -- Partition   

  Pivot := Table(Table'First);    -- define leftmost element as the pivot   

  -- Find and exchange values that are out of place.   
  Up :=   Table'First;               -- set Up to point to leftmost element   
  Down := Table'Last;           -- set Down to point to rightmost element   

  LOOP 
    -- Move Up to the next value larger than Pivot.   
    WHILE (Table(Up) <= Pivot) AND (Up < Table'Last) LOOP
      Up := Up + 1;
    END LOOP;
    -- assertion:  Table(Up) > Pivot or Up is equal to Table'Last

    -- Move Down to the next value less than or equal to Pivot.   
    WHILE Table(Down) > Pivot LOOP
      Down := Down - 1;
    END LOOP;
    -- assertion:  Table(Down) <= Pivot   

    -- Exchange out of order values.   
    IF Up < Down THEN
      Exchange (Table(Up), Table(Down));
    END IF;
  
    EXIT WHEN Up >= Down;       -- until Up meets or passes Down   
  END LOOP;
  -- Assertion: values <= Pivot have subscripts <= Down and
  --  values > Pivot have subscripts > Down
   
  -- Put pivot value where it belongs and define PivIndex.   
  Exchange (Table(Table'First), Table(Down));
  PivIndex := Down;

END Partition;
