PROCEDURE BuildList (Head : IN OUT ListPointer) IS

-- Builds a linked list of strings pointed to by Head.
-- Each new string is appended to the end of the list so
-- the strings will be stored in the order in which they were read.
-- Pre : None
-- Post: Head points to the first string entered.  Head is
-- set to NULL if the sentinel string is the first string.

  Sentinel : CONSTANT String3 :=  "***";
  NextWord : String3;     -- data word
  Last     : ListPointer; -- always points to last node in list

BEGIN -- BuildList

  -- Display instructions to user.
  Text_IO.Put(Item => "Enter each data string on a line.");

  Text_IO.New_Line;
  Text_IO.Put(Item => "Enter ");
  Text_IO.Put(Item => Sentinel);
  Text_IO.Put(Item => " when done.");
  Text_IO.New_Line;

  -- Create and fill the list head.
  Text_IO.Get(Item => NextWord);    -- read first word
  IF NextWord = Sentinel THEN
    Head := NULL;                   -- empty list
  ELSE
    Head := NEW ListNode;           -- create the list head
    Head.Word := NextWord;          -- store it in list head
    Text_IO.Get(Item => NextWord);
    
    -- Now build the rest of the list
    Last := Head;
    WHILE NextWord /= Sentinel LOOP
    -- invariant:
    --  Last points to the last node in a list and
    --  the last string read is stored in node Last.ALL and
    --  no prior data string was the Sentinel

      Last.Link := NEW ListNode;      -- attach a new node to Last.ALL
      Last      := Last.Link;         -- reset Last to new list end
      Last.Word := NextWord;          -- store last word read
      Text_IO.Get(Item => NextWord);  -- read next word
    END LOOP;
    -- assert: The last string read was the Sentinel
  END IF;

END BuildList;

