SEPARATE(GenericTables)
PROCEDURE Delete (Table   : IN OUT TableType;
		  Target  : KeyType;
		  Success : OUT Boolean) IS

  --  Deletes the element with key Target from an ordered list.
  --  Pre : Table points to the header node for an ordered list.
  --  Post: Success is True if deletion is performed; Success is False
  --        if deletion is not performed because there is no element
  --        whose key is Target.

  Previous : NodePointer;  -- pointer to last list key < ElKey
  Next     : NodePointer;  -- pointer to first list key >= ElKey
  Temp     : NodePointer;  -- temporary variable
  TempFlag : Boolean;      -- hold temporary success flag

BEGIN -- Delete

  IF Table.Head = NULL THEN
    RAISE UninitializedTable;
  END IF;

  -- Search the list for the key Target.
  Previous := Table.Head;  -- Previous points to first dummy node
  Locate (Target, Previous, Next, TempFlag);
  Success := TempFlag;

  -- If Target is found, delete it.
  IF TempFlag THEN
    Temp := Next;               -- point Temp to the node being deleted
    Previous.Link := Next.Link; -- disconnect node Next
    Dispose (Temp);             -- deallocate storage
    Table.NumItems := Table.NumItems - 1;
  END IF;

END Delete;

