WITH Unchecked_Deallocation;
PACKAGE BODY GenericTables IS

-- Implementation of the abstract data type for a table of
-- element records, each containing a key.
-- This implementation uses one-way linked lists.

-- We need to instantiate Unchecked_Deallocation to get a Dispose
  PROCEDURE Dispose IS NEW Unchecked_Deallocation
    (Object => ListNode, Name => NodePointer);

-- local procedure, not exported to client
  PROCEDURE Locate(Target       : KeyType;
		   Previous     : IN OUT NodePointer;
		   Next         : OUT NodePointer;
		   SearchSuccess: OUT Boolean) IS SEPARATE;

-- Operators

  FUNCTION SizeOfTable (Table : TableType) RETURN Natural IS SEPARATE;

  PROCEDURE CreateTable (Table : IN OUT TableType) IS SEPARATE;

  PROCEDURE Search (Table   : TableType;
		    Target  : KeyType;
		    Success : OUT Boolean) IS SEPARATE;

  PROCEDURE Insert (Table   : IN OUT TableType;
		    El      : Element;
		    Success : OUT Boolean) IS SEPARATE;

  PROCEDURE Delete (Table   : IN OUT TableType;
		    Target  : KeyType;
		    Success : OUT Boolean) IS SEPARATE;

  PROCEDURE Replace (Table   : IN OUT TableType;
		     El      : Element;
		     Success : OUT Boolean) IS SEPARATE;

  PROCEDURE Retrieve (Table   : TableType;
		      Target  : KeyType;
		      El      : OUT Element;
		      Success : OUT Boolean) IS SEPARATE;

  PROCEDURE Traverse (Table : TableType) IS SEPARATE;

END GenericTables;

