SEPARATE (GenericTables)
PROCEDURE Locate(Target       : KeyType;
		 Previous     : IN OUT NodePointer;
		 Next         : OUT NodePointer;
		 SearchSuccess: OUT Boolean) IS

--  Attempts to locate a node with key value Target in the
--  list whose first node is pointed to by Previous.
--  Pre : Target is defined; Previous points to the first dummy node.
--  Post: If Target is located, SearchSuccess is set to True;
--        otherwise, SearchSuccess is set to False.
--        Previous points to the last list node with key < Target;
--        Next points to the first list node with key >= Target.

  CurrentKey : KeyType;        -- key of node Next.ALL
  Current : NodePointer;       -- keeps track of current node
			       -- necessary because Next is OUT

BEGIN -- Locate

  -- Search for first node with key >= Target.
  -- Start with first actual node.
  Current := Previous.Link;
  ExtractKey (Current.Item, CurrentKey);     -- get first key
  WHILE Bigger(Target, CurrentKey) LOOP
    -- invariant:
    --   Target > key of each node pointed to by Current so far.
        
    Previous := Current;                   -- advance Previous
    Current  := Current.Link;              -- advance Next
    ExtractKey (Current.Item, CurrentKey);    -- get next key
  END LOOP;
  -- assert: Target is located or CurrentKey is larger than Target.

  -- Set Next and flag to indicate search results.
  Next := Current;
  SearchSuccess := NOT Bigger(CurrentKey, Target);

END Locate;

