SEPARATE(GenericTables)
PROCEDURE Insert (Table   : IN OUT TableType;
		  El      : Element;
		  Success : OUT Boolean) IS

--  Inserts item El into an ordered list.
--  Pre : Table points to the header node for an ordered list.
--  Post: Success is True if insertion is performed; Success is False
--        if insertion is not performed because there is already
--        an element with the same key as El.
    
  Previous      : NodePointer;  -- pointer to node preceding El
  Next          : NodePointer;  -- pointer to node following El
  SearchSuccess : Boolean;      -- search result
  ElKey         : KeyType;      -- key of record El

BEGIN -- Insert

  IF Table.Head = NULL THEN
    RAISE UninitializedTable;
  END IF;

  -- Validate ElKey and search for a valid key.
  ExtractKey (El, ElKey);
  IF Bigger(MinKey, ElKey) OR Bigger(ElKey, MaxKey) THEN
    Success := False;               -- ElKey is out of range
  ELSE
    -- Search the list for ElKey.
    Previous := Table.Head;          -- start at first dummy node
    Locate (ElKey, Previous, Next, SearchSuccess);
  END IF;

  -- Insert if ElKey is in range and is a new key
  IF NOT SearchSuccess THEN
    Success := NOT SearchSuccess;   -- ElKey is new if search fails
    Previous.Link := NEW ListNode;  -- join new node to Previous.ALL
    Previous.Link.Link := Next;     -- join new node to Next.ALL
    Previous.Link.Item := El;       -- store El in new node
    Table.NumItems := Table.NumItems + 1;
  END IF;

END Insert;

