WITH Text_IO;
WITH My_Int_IO;
WITH My_Flt_IO;
PACKAGE BODY RobustInput IS

  PROCEDURE Get (Item : OUT Integer; 
                 MinVal : IN Integer; 
                 MaxVal : IN Integer) IS
  
  -- Gets an integer value in the range MinVal..MaxVal from the terminal
  -- Pre: MinVal and MaxVal are defined
  -- Post: MinVal <= Item <= MaxVal
       
    SUBTYPE TempType IS Integer RANGE MinVal..MaxVal;
    TempItem : TempType;     -- temporary copy of MinVal   

  BEGIN -- Get
            
    LOOP
      BEGIN      -- exception handler block
        Text_IO.Put(Item => "Enter an integer between ");
        My_Int_IO.Put(Item => MinVal, Width => 0);  
        Text_IO.Put(Item => " and ");
        My_Int_IO.Put(Item => MaxVal, Width => 0);  
        Text_IO.Put(Item => " > ");
        My_Int_IO.Get(Item => TempItem);
        Item := TempItem;
        EXIT;    -- valid data
      EXCEPTION  -- invalid data
        WHEN Constraint_Error =>
          Text_IO.Put ("Value entered is out of range. Please try again.");
          Text_IO.New_Line;
          Text_IO.Skip_Line;
        WHEN Text_IO.Data_Error =>
          Text_IO.Put ("Value entered not an integer. Please try again.");
          Text_IO.New_Line;
          Text_IO.Skip_Line;
      END;       -- exception handler block
    END LOOP;
    -- assert: Item is in the range MinVal to MaxVal   

  END Get;

  PROCEDURE Get (Item : OUT Float; 
                 MinVal : IN Float; 
                 MaxVal : IN Float) IS
       
  -- Gets a float value in the range MinVal..MaxVal from the terminal
  -- Pre: MinVal and MaxVal are defined
  -- Post: MinVal <= Item <= MaxVal

    SUBTYPE TempType IS Float RANGE MinVal..MaxVal;
    TempItem : TempType;     -- temporary copy of MinVal   

  BEGIN -- Get
            
    LOOP
      BEGIN       -- exception handler block
        Text_IO.Put(Item => "Enter a floating-point value between ");
        My_Flt_IO.Put(Item => MinVal, Fore=> 1, Aft => 2, Exp => 0);  
        Text_IO.Put(Item => " and ");
        My_Flt_IO.Put(Item => MaxVal, Fore=> 1, Aft => 2, Exp => 0);  
        Text_IO.Put(Item => " > ");
        My_Flt_IO.Get(Item => TempItem);
        Item := TempItem;
        EXIT;     -- valid data
      EXCEPTION   -- invalid data
        WHEN Constraint_Error =>
          Text_IO.Put ("Value entered is out of range. Please try again.");
          Text_IO.New_Line;
          Text_IO.Skip_Line;
        WHEN Text_IO.Data_Error =>
          Text_IO.Put ("Value entered not floating point. Please try again.");
          Text_IO.New_Line;
          Text_IO.Skip_Line;
      END;        -- exception handler block
    END LOOP;
    -- assert: Item is in the range MinVal to MaxVal   

  END Get;

END RobustInput;
