WITH Text_IO;
WITH My_Int_IO;
WITH My_Flt_IO;
PROCEDURE GrowMoney IS
 
-- Displays a table of interest earned and account balance for each
-- investment year for fixed or varying rate certificates.
 
  SUBTYPE NonNegFloat IS Float RANGE 0.0 .. Float'Last;
 
  FixedOrVar : Character;   -- input - indicates fixed or varying rate
  Deposit :    NonNegFloat; -- input - initial amount of deposit   
  Rate :       NonNegFloat; -- input - annual rate of interest   
  TargetBal :  NonNegFloat; -- input - the target certificate amount   
  Balance :    NonNegFloat; -- output - current certificate amount   
  Interest :   NonNegFloat; -- output - amount of annual interest   
  Year :       Natural;     -- output - year of investment  
 
BEGIN -- GrowMoney  

  Text_IO.Put (Item => "Enter the deposit amount $");
  My_Flt_IO.Get (Item => Deposit);  
 
  Text_IO.Put (Item => "Enter the desired final balance $");
  My_Flt_IO.Get (Item => TargetBal);  
 
  Text_IO.Put (Item => "Is the interest rate fixed (F) or variable (V)? ");
  Text_IO.Get (Item => FixedOrVar);  

  IF FixedOrVar = 'F' THEN
    Text_IO.Put (Item => "Enter the interest rate as a decimal fraction > ");
    My_Flt_IO.Get (Item => Rate);  
  END IF;
 
  -- Display table heading   
  Text_IO.New_Line;
  Text_IO.Put (Item => "Year       Interest        Balance");  
  Text_IO.New_Line;
 
  -- Display the certificate balance for each year.   
  Year := 0;
  Balance := Deposit;
  Interest := 0.0;
  WHILE Balance < TargetBal LOOP
     -- invariant:
     --   Balance < TargetBal + Interest and
     --   Balance is the sum of Deposit and all values of Interest

    Year := Year + 1;
    IF FixedOrVar = 'V' THEN
      Text_IO.Put (Item => "Enter rate for year ");
      My_Int_IO.Put (Item => Year, Width => 0);  
      Text_IO.Put (Item => " > ");
      My_Flt_IO.Get (Item => Rate);  
    END IF;
 
    Interest := Balance * Rate;
    Balance := Balance + Interest;
 
    My_Int_IO.Put (Item => Year, Width => 4);      
    My_Flt_IO.Put (Item => Interest, Fore => 12, Aft => 2, Exp => 0);
    My_Flt_IO.Put (Item => Balance, Fore => 12, Aft => 2, Exp => 0);  
    Text_IO.New_Line;
  END LOOP;
  -- assert: Balance >= TargetBal and
  --   Balance is the sum of Deposit and all values of Interest
     
  Text_IO.New_Line;
  Text_IO.Put (Item => "Certificate amount reaches target after ");
  My_Int_IO.Put (Item => Year, Width => 2);
  Text_IO.Put (Item => " years");  
  Text_IO.New_Line;
  Text_IO.Put (Item => "Final balance is $");
  My_Flt_IO.Put (Item => Balance, Fore => 1, Aft => 2, Exp => 0);  
  Text_IO.New_Line;

END GrowMoney;
