WITH Text_IO;
WITH My_Int_IO;
WITH RobustInput;
WITH Math; USE Math;
PROCEDURE PrimeNumber IS

-- Displays the smallest divisor (other than 1) of the positive N if
-- a divisor exists; otherwise, displays a message that N is prime.  

  MaxN : CONSTANT Positive := 1000;             -- largest n to be tested   
  SUBTYPE SmallPos IS Positive RANGE 2..MaxN;   -- range of values for n   

  N :              Positive;   -- number being tested as a prime   
  FirstDiv :       SmallPos;   -- first divisor if found   
  MaxPossibleDiv : SmallPos;   -- truncated square root of N
  Prime    :       Boolean;    -- flag -- signals whether N is prime
                               -- (True) or not prime (False)        

BEGIN  -- PrimeNumber   

  -- Enter an integer to test for a prime number   
  Text_IO.Put(Item => "Enter a number that you think is a prime.");
  Text_IO.New_Line;
  RobustInput.Get (Item => N, MinVal => 2, MaxVal => MaxN);
  
  -- Find smallest divisor FirstDiv or determine that N is prime.
  -- Set Prime to indicate whether or not N is a prime number.
    
  -- Finds first divisor (FirstDiv) of N if it exists.
  -- Pre:  N is assigned a value.
  -- Post: FirstDiv is the first divisor of N besides 1 and N.
        -- Prime is True if a divisor is not found; otherwise,
        -- Prime is False.
     
  IF N = 2 THEN
    Prime := True;                 -- 2 is a prime number   
  ELSIF (N REM 2) = 0 THEN         -- N is even
    Prime := False;
    FirstDiv := 2;                 -- 2 is first divisor of even N
  ELSE                             -- N is odd   
    Prime := True;                 -- Assume that N is prime   
    MaxPossibleDiv := Positive(Sqrt(Float(N)) - 0.5);
    FirstDiv := 3;                 -- Try 3 first   

    WHILE Prime AND (FirstDiv <= MaxPossibleDiv) LOOP
    -- invariant:
    -- FirstDiv during pass i is 1 + 2 * i (3, 5, 7, ... ) and
    -- No prior value of FirstDiv is a divisor of N and
    -- FirstDiv is less than or equal to the square root of N
            
      IF N REM FirstDiv = 0 THEN   -- N is not prime
        Prime := False ;
      ELSE
        FirstDiv  := FirstDiv + 2; -- Try next odd number   
      END IF;
    END  LOOP;
    -- assertion:
    -- Prime is True and FirstDiv > Sqrt(N) or
    -- Prime is False and FirstDiv is the smallest divisor of N
          
  END IF;
   
  -- Display first divisor or a message that N is prime   
  IF Prime THEN
    My_Int_IO.Put(Item => N, Width => 5);
    Text_IO.Put(Item => " is a prime number");
    Text_IO.New_Line;
  ELSE
    My_Int_IO.Put(Item => FirstDiv, Width => 5);
    Text_IO.Put(Item => " is the smallest divisor of ");
    My_Int_IO.Put(Item => N, Width => 1);
    Text_IO.New_Line;
  END IF;

END PrimeNumber;
