WITH Text_IO;
PACKAGE BODY NumToWord IS

-- package body for displaying nonnegative integers in words

  SUBTYPE Digit IS Natural RANGE 0..9;

-- local procedures

  PROCEDURE Put1Digit (Item : Digit) IS
    
  -- Puts its argument in words.
  -- Pre:  Item is assigned a value between 0 and 9.
  -- Post: Item is displayed in words.
    
  BEGIN -- Put1Digit   

    CASE Item IS
      WHEN 0 => Text_IO.Put (Item => "zero");
      WHEN 1 => Text_IO.Put (Item => "one");
      WHEN 2 => Text_IO.Put (Item => "two");
      WHEN 3 => Text_IO.Put (Item => "three");
      WHEN 4 => Text_IO.Put (Item => "four");
      WHEN 5 => Text_IO.Put (Item => "five");
      WHEN 6 => Text_IO.Put (Item => "six");
      WHEN 7 => Text_IO.Put (Item => "seven");
      WHEN 8 => Text_IO.Put (Item => "eight");
      WHEN 9 => Text_IO.Put (Item => "nine");
    END CASE;

  END Put1Digit;


  PROCEDURE Call1Digit (Units : Digit) IS
       
  -- Calls procedure Put1Digit with parameter Units if
  -- Units is not zero.
  -- Pre:  Units is assigned a value between 0 and 9.
  -- Post: Calls Put1Digit if Units is not 0.
       
  BEGIN -- Call1Digit   
    IF Units /= 0 THEN
      Text_IO.Put(Item => " ");
      Put1Digit (Item => Units);
    END IF;
  END Call1Digit;

  PROCEDURE Put2Digits (Item : Natural16) IS
    
  -- Puts its argument in words.
  -- Pre:  Item is assigned a value between 0 and 99.
  -- Post: Item is displayed in words.
  -- Uses: Put1Digit

    Tens :  Digit;                             -- tens digit   
    Units : Digit;                             -- units digit   

  BEGIN -- Put2Digits   

    Tens  := Item / 10;                -- Get tens digit   
    Units := Item REM 10;              -- Get units digit   

    CASE Tens IS
      WHEN 0 => 
        Put1Digit (Units);               -- less than ten   
      WHEN 1 => 
        CASE Units IS                     -- in the teens   
            WHEN 0 => Text_IO.Put (Item => "ten");
            WHEN 1 => Text_IO.Put (Item => "eleven");
            WHEN 2 => Text_IO.Put (Item => "twelve");
            WHEN 3 => Text_IO.Put (Item => "thirteen");
            WHEN 5 => Text_IO.Put (Item => "fifteen");
            WHEN 8 => Text_IO.Put (Item => "eighteen");
            WHEN 4 | 6 | 7 | 9 => 
              Put1Digit (Units);  -- Put  ...teen   
              Text_IO.Put (Item => "teen");
        END CASE;
      WHEN 2 => 
        Text_IO.Put (Item => "twenty");        -- Put twenty ...   
        Call1Digit (Units);
      WHEN 3 => 
        Text_IO.Put (Item => "thirty");        -- Put thirty ...   
        Call1Digit (Units);
      WHEN 4 => 
        Text_IO.Put (Item => "forty");         -- Put forty ...    
        Call1Digit (Units);
      WHEN 5 => 
        Text_IO.Put (Item => "fifty");         -- Put fifty ...    
        Call1Digit (Units);
      WHEN 8 => 
        Text_IO.Put (Item => "eighty");        -- Put eighty ...   
        Call1Digit (Units);
      WHEN 6 | 7 | 9 => 
        Put1Digit (Tens);      -- Put ...ty ...   
        Text_IO.Put (Item => "ty");
        Call1Digit (Units);
    END CASE;
  END Put2Digits;

  PROCEDURE PutInWords (Item: Natural16) IS

    CopyOfItem: Natural;
    Thousands:  Natural;
    Hundreds:   Natural;

  BEGIN

    IF Item = 0 THEN 
      Put1Digit (Item => Item);
    ELSE
      CopyOfItem := Item;
      IF CopyOfItem >= 1000 THEN
        Thousands := CopyOfItem / 1000;
        Put2Digits (Item => Thousands);
        Text_IO.Put (Item => " thousand ");
        CopyOfItem := CopyOfItem REM 1000;
      END IF;

      IF CopyOfItem >= 100 THEN
        Hundreds := CopyOfItem / 100;
        Put1Digit (Item => Hundreds);
        Text_IO.Put (Item => " hundred ");
        CopyOfItem := CopyOfItem REM 100;
      END IF;

      IF CopyOfItem /= 0 THEN
        Put2Digits (Item => CopyOfItem);
      END IF;
    END IF;

  END PutInWords;
      
END NumToWord;

