WITH Text_IO;
WITH NumToWord;
WITH RobustInput;
PROCEDURE DisplayCheck IS

-- Displays a check amount in words.
-- Pre:  Check is read from the terminal; amount must be in 0.00 .. 32767.99
-- Post: Displays the value of Check in words.
-- Uses: NumToWord.PutInWords; Text_IO.Put; Text_IO.New_Line;
--       RobustInput.Get;
  
  MaxCheck : CONSTANT Float := 32767.00;

  SUBTYPE CheckAmount IS Float RANGE 0.0 .. MaxCheck;

  Check :        CheckAmount;             -- amount of check
  Dollars :      NumToWord.Natural16;     -- the dollar amount   
  Cents :        NumToWord.Natural16;     -- the cents amount   

BEGIN -- DisplayCheck   

  Text_IO.Put (Item => "Please enter check amount");
  Text_IO.New_Line;
  RobustInput.Get (Item => Check, 
    MinVal => CheckAmount'First, MaxVal => CheckAmount'Last);
    
  Dollars := NumToWord.Natural16(Check);  -- round downward
  IF Float(Dollars) > Check THEN          -- in case rounding was upward
     Dollars := Dollars - 1;
  END IF;

  Cents := NumToWord.Natural16(100.0 * (Check - Float(Dollars)));

  NumToWord.PutInWords (Item => Dollars); -- Display dollar amount   
  Text_IO.Put (Item => " dollar");
  IF Dollars /= 1 THEN
    Text_IO.Put (Item => "s");
  END IF;
  Text_IO.Put (Item => " and ");

  NumToWord.PutInWords (Item => Cents);   -- Display cents amount
  Text_IO.Put (Item => " cent");
  IF Cents /= 1 THEN
    Text_IO.Put (Item => "s");
  END IF;
  Text_IO.New_Line;
  
END DisplayCheck;
