WITH Text_IO;
PROCEDURE GetNaturalToken (NumData : OUT Natural) IS
--
-- Reads consecutive characters ending with the symbol %.  Computes
-- the integer value of the digit characters, ignoring non-digits.
-- Pre:  None
-- Post: NumData is the value of the digit characters read.

  Base :     CONSTANT Positive := 10;   -- the number system base   
  Sentinel : CONSTANT Character := '%'; -- the sentinel character   

  TempNum :  Natural;                   -- to compute the numerical value
  Next :     Character;                 -- each character read   
  Digit :    Natural;                   -- the value of each numeric character 
                                        -- (its ASCII position)
BEGIN -- GetNaturalToken   

  -- Accumulate the numeric value of the digits in TempNum   
  TempNum := 0;                         -- initial value is zero   
  Text_IO.Get(Item => Next);            -- Read first character   
  WHILE Next /= Sentinel LOOP
    -- invariant:
    --   No prior value of Next is the sentinel and
    --   if Next is a digit, TempNum is multiplied by Base and
    --   Next's digit value is added to TempNum
      
    IF (Next >= '0') AND (Next <= '9') THEN
      -- Process digit   
      Digit := Character'Pos(Next) - Character'Pos('0');  -- Get digit value   
      TempNum := Base * TempNum + Digit; -- Add digit value   
    END IF;
    Text_IO.Get(Item => Next);           -- Read next character   
  END  LOOP;
  -- assert:
  --   Next is the sentinel and
  --   TempNum is the number in base Base formed from the digit
  --   characters read as data

  NumData := TempNum;

END GetNaturalToken;

