WITH Text_IO;
PROCEDURE Cryptogram IS

  SUBTYPE Letter IS Character RANGE 'A'..'Z';
  TYPE CodeArray IS ARRAY (Letter) OF Character;
  Code      : CodeArray;                  -- array of code symbols   

  FUNCTION Cap (InChar : Character) RETURN Character IS

  -- returns an upper-case letter
  -- Pre: InChar is defined
  -- Post: if InChar is a lower-case letter, returns its upper-case
  --   equivalent; otherwise, returns InChar unmodified

  BEGIN -- Cap

    IF InChar IN 'a'..'z' THEN
      RETURN Character'Val(Character'Pos(InChar)
            - Character'Pos('a') + Character'Pos('A'));
    ELSE
      RETURN InChar;
    END IF;

  END Cap;


  PROCEDURE ReadCode (Code : OUT CodeArray) IS
  
  -- Reads in the code symbol for each letter.
  -- Pre : None
  -- Post: 26 code symbols are read into array Code.
    
  BEGIN -- ReadCode   

    Text_IO.Put(Item => "Enter a code symbol under each letter.");
    Text_IO.New_Line;
    Text_IO.Put(Item => "ABCDEFGHIJKLMNOPQRSTUVWXYZ");  
    Text_IO.New_Line;

    -- Read each code symbol into array Code.   
    FOR NextLetter IN Letter LOOP
      Text_IO.Get(Item => Code(NextLetter));
    END LOOP;
    Text_IO.Skip_Line;

  END ReadCode;


  PROCEDURE Encrypt (Code : CodeArray) IS
  
  -- Reads a plaintext (unencoded) message and displays its coded version.
  -- Pre : The code for letter i is saved in Code(i).
  -- Post: Displays the encoded message
    
    SUBTYPE Line IS String(1..80);

    PlainText : Line;
    CodedText : Line;
    HowLong   : Natural;

  BEGIN -- Encrypt   

    Text_IO.Put(Item => "Enter each character of your message.");
    Text_IO.New_Line;
    Text_IO.Put(Item => "No more than 80 characters, please.");
    Text_IO.New_Line;
    Text_IO.Put(Item => "Enter a CR after your message.");
    Text_IO.New_Line;

    Text_IO.Get_Line (Item => PlainText, Last => HowLong);
    FOR WhichChar IN 1..HowLong LOOP
      IF Cap(PlainText(WhichChar)) IN Letter THEN
        CodedText(WhichChar) := Code(Cap(PlainText(WhichChar)));
      ELSE
        CodedText(WhichChar) := PlainText(WhichChar);
      END IF;
    END LOOP;
    Text_IO.Put (Item => CodedText(1..HowLong));
    Text_IO.New_Line;

  END Encrypt;

BEGIN -- Cryptogram   

  -- Read in the code symbol for each letter.   
  ReadCode (Code);

  -- Read each character and print the cryptogram   
  Encrypt (Code);

END Cryptogram;

