WITH Text_IO;
WITH My_Int_IO;
WITH SimpleDates;
PROCEDURE SortScoreFile IS

-- Sorts and displays an array of test score records
-- The records are read from a file SCORES.DAT

  MaxSize : CONSTANT Positive := 250;
  MaxScore : CONSTANT Positive := 100;

  SUBTYPE StudentName IS String(1..20);
  SUBTYPE ClassIndex IS Positive RANGE 1..MaxSize;
  SUBTYPE ClassRange IS Natural  RANGE 0..MaxSize;
  SUBTYPE ScoreRange IS Natural  RANGE 0..MaxScore;

  TYPE ScoreRecord IS RECORD
     Name:  StudentName;
     Score: ScoreRange;
  END RECORD;

  TYPE ScoreArray IS ARRAY (ClassIndex) OF ScoreRecord;

  Scores : ScoreArray;
  ClassSize : ClassRange;

  PROCEDURE GetRecords(Scores: OUT ScoreArray; ClassSize: OUT ClassRange) IS
    
    TestScores: Text_IO.File_Type; -- program variable naming the input file
    TempSize:   ClassRange;
    TempRecord: ScoreRecord;

  BEGIN  -- GetRecords 

    -- Open the file and associate it with the file variable name
    Text_IO.Open
      (File => TestScores, Mode => Text_IO.In_File, Name => "SCORES.DAT");

    -- Read each data item
    -- and store it in the appropriate element of Scores

    TempSize := 0;                         -- initial class size   
    -- Read each array element until done.   
    WHILE (NOT Text_IO.End_Of_File(TestScores)) AND (TempSize < MaxSize) LOOP
      -- invariant:
      --   Records remain in the file and
      --   TempSize <= MaxSize
        
      Text_IO.Get(File => TestScores, Item => TempRecord.Name);
      My_Int_IO.Get(File => TestScores, Item => TempRecord.Score);

      TempSize := TempSize + 1;
      Scores(TempSize) := TempRecord;     -- Save the score   

    END LOOP;      
    -- assert:
    --   End of file reached or
    --   TempSize is MaxSize
      
    IF TempSize = MaxSize THEN
      Text_IO.Put(Item => "Array is filled.");  
      Text_IO.New_Line;
    END IF;

    ClassSize := TempSize;

  END GetRecords;


  PROCEDURE Exchange(Student1, Student2: IN OUT ScoreRecord) IS

    TempRecord: ScoreRecord;

  BEGIN

    TempRecord := Student1;
    Student1 :=   Student2;
    Student2 :=   TempRecord;

  END Exchange;


  PROCEDURE SelectSort(Scores: IN OUT ScoreArray; ClassSize: IN ClassRange) IS

    IndexOfMax: ClassRange;

  BEGIN

    FOR PositionToFill IN REVERSE 2..ClassSize LOOP

      -- Find the element in subarray 1..PositionToFill with largest Score
      IndexOfMax := PositionToFill;
      FOR ItemToCompare IN REVERSE 1..PositionToFill-1 LOOP
        IF Scores(ItemToCompare).Score > Scores(IndexOfMax).Score THEN
          IndexOfMax := ItemToCompare;
        END IF;
      END LOOP;
      -- assert: element at IndexOfMax is largest in subarry
      IF IndexOfMax /= PositionToFill THEN
        Exchange(Scores(PositionToFill),Scores(IndexOfMax));
      END IF;

    END LOOP;

  END SelectSort;


  PROCEDURE DisplayScores(Scores: ScoreArray; ClassSize: ClassRange) IS

  BEGIN

    FOR I IN 1..ClassSize LOOP
      My_Int_IO.Put(Item => I, Width => 3);
      Text_IO.Put(Item => "  ");
      Text_IO.Put(Item => Scores(I).Name);
      My_Int_IO.Put(Item => Scores(I).Score, Width => 4);
      Text_IO.New_Line;
    END LOOP;

  END DisplayScores;
  

BEGIN -- SortScoreFile

  Text_IO.Put(Item => "Today is ");
  SimpleDates.Put(Item => SimpleDates.Today);
  Text_IO.New_Line;

  GetRecords(Scores => Scores, ClassSize => ClassSize);
  Text_IO.Put(Item => "Original Test File:");
  Text_IO.New_Line;
  Text_IO.New_Line;
  DisplayScores(Scores => Scores, ClassSize => ClassSize);

  SelectSort(Scores => Scores, ClassSize => ClassSize);
  Text_IO.New_Line;
  Text_IO.Put(Item => "Sorted Test File:");
  Text_IO.New_Line;
  Text_IO.New_Line;
  DisplayScores(Scores => Scores, ClassSize => ClassSize);
  Text_IO.New_Line;

END SortScoreFile;
