WITH Text_IO;
WITH My_Int_IO;
WITH My_Flt_IO;
PROCEDURE CheckBook IS

 -- Reads the starting balance for a checking account and processes
 -- all transactions.  Prints the new balance after each transaction
 -- is processed.  Also prints a count of the total number of checks
 -- and deposits processed.
  
  StartBal  : Float;             -- input - starting balance   
  CurBal    : Float;             -- output - current balance   
  NumCheck  : Natural;           -- output - number of checks   
  NumDep    : Natural;           -- output - number of deposits   

  PROCEDURE Instruct (StartBal : OUT Float) IS
  
  -- Displays the instructions and reads the starting balance.
  -- Pre:  None
  -- Post: User instructions are displayed and StartBal is read in.
    
  BEGIN  -- Instruct   

    Text_IO.Put(Item => "Balance your checking account!");
    Text_IO.New_Line;
    Text_IO.New_Line;
    Text_IO.Put(Item => "Enter C (Check), D (Deposit), or Q (Quit)");
    Text_IO.New_Line;
    Text_IO.Put(Item => "after prompt C, D, or Q: ");
    Text_IO.New_Line;
    Text_IO.Put(Item => "Enter a positive number after prompt Amount $");
    Text_IO.New_Line;  
    Text_IO.New_Line;
    Text_IO.Put(Item => "Begin by entering your starting balance $");
    My_Flt_IO.Get(Item => StartBal);

  END Instruct;


  PROCEDURE Process (StartBal : IN Float;
                     CurBal   : IN OUT Float;
                     NumCheck : IN OUT Natural;
                     NumDep   : IN OUT Natural) IS
  
  -- Processes each transaction.  Reads each transaction, updates and
  -- prints the current balance and increments the count of checks or
  -- deposits.
  -- Pre:  StartBal is assigned a value.
  -- Post: CurBal is StartBal plus deposits and minus withdrawals.
  --       NumCheck is the count of checks.
  --       NumDep is the count of deposits.
  -- Uses: ReadTran, Update, and DisplayTran
    
    TranType : Character;    -- transaction type (check or deposit)   
    Amount : Float;          -- transaction amount   

  BEGIN  -- Process stub 

    Text_IO.Put(Item => "Procedure Process entered.");
    Text_IO.New_Line;
    CurBal := StartBal;
    NumCheck := 0;
    NumDep := 0;

  END Process;


  PROCEDURE Report (StartBal : IN Float;
                    CurBal   : IN Float;
                    NumCheck : IN Natural;
                    NumDep   : IN Natural) IS
  
  -- Prints the starting and final balances and the count of checks
  -- and deposits.
  -- Pre:  StartBal, CurBal, NumCheck, and NumDep are assigned values.
  -- Post: Program results are displayed.
    
  BEGIN  -- Report   

    Text_IO.New_Line;
    Text_IO.Put(Item => "Starting balance was $");
    My_Flt_IO.Put(Item => StartBal, Fore=>1, Aft=>2, Exp=>0);
    Text_IO.New_Line;
    Text_IO.Put(Item => "Final    balance is  $");
    My_Flt_IO.Put(Item => CurBal  , Fore=>1, Aft=>2, Exp=>0);
    Text_IO.New_Line;
    Text_IO.Put(Item => "Number of checks written: ");
    My_Int_IO.Put(Item => NumCheck, Width=>1);  
    Text_IO.New_Line;
    Text_IO.Put(Item => "Number of deposits made : ");
    My_Int_IO.Put(Item => NumDep, Width=>1);  
    Text_IO.New_Line;

  END Report;


BEGIN  -- CheckBook  
  -- Display user instructions and read StartBal  
  Instruct (StartBal);

  -- Process each transaction  
  Process (StartBal, CurBal, NumCheck, NumDep);

  -- Print starting and final balances and count of checks/deposits  
  Report (StartBal, CurBal, NumCheck, NumDep);

END CheckBook;
