  -- specifications of lower-level procedures used by Process

  PROCEDURE ReadTran (TranType : IN OUT Character; Amount   : OUT Float);

  PROCEDURE Update (TranType : IN Character;
                    Amount   : IN Float;
                    CurBal   : IN OUT Float;
                    NumCheck : IN OUT Natural;
                    NumDep   : IN OUT Natural);

  PROCEDURE DisplayTran (TranType : IN Character;
                         Amount   : IN Float;
                         CurBal   : IN Float);

  PROCEDURE Process (StartBal : IN Float;
                     CurBal   : IN OUT Float;
                     NumCheck : IN OUT Natural;
                     NumDep   : IN OUT Natural) IS

  -- Processes each transaction.  Reads each transaction, updates and
  -- prints the current balance and increments the count of checks or
  -- deposits.
  -- Pre:  StartBal is assigned a value.
  -- Post: CurBal is StartBal plus deposits and minus withdrawals.
  --       NumCheck is the count of checks.
  --       NumDep is the count of deposits.
  -- Uses: ReadTran, Update, and DisplayTran

    TranType : Character;    -- transaction type (check or deposit)
    Amount : Float;          -- transaction amount

  BEGIN  -- Process

    -- Initialize counters to zero and CurBal to StartBal
    NumCheck := 0;
    NumDep := 0;
    CurBal := StartBal;

    -- Read first transaction
    ReadTran (TranType, Amount);

    -- Process each transaction until done
    WHILE TranType /= Sentinel LOOP
      Update (TranType, Amount, CurBal, NumCheck, NumDep);
      DisplayTran (TranType, Amount, CurBal);
      ReadTran (TranType, Amount);
    END LOOP;

  END Process;
